CutterDensityHeightPickupFix = {
	prerequisitesPresent = function (specializations)
		return SpecializationUtil.hasSpecialization(Cutter, specializations)
	end
}

function CutterDensityHeightPickupFix.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "processPickupCutterArea", CutterDensityHeightPickupFix.processPickupCutterArea)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getAllowCutterAIFruitRequirements", CutterDensityHeightPickupFix.getAllowCutterAIFruitRequirements)
end

function CutterDensityHeightPickupFix.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", CutterDensityHeightPickupFix)
end

function CutterDensityHeightPickupFix:onLoad(savegame)
	local spec = self.spec_cutter

	if spec.fruitTypes ~= nil and self.addAIFruitRequirement ~= nil then
		for fruitType, state in pairs(spec.fruitTypes) do
			if state then
				self:addAIFruitRequirement(fruitType, 0, g_currentMission.terrainDetailHeightTypeNumChannels)
			end
		end

		self:setAIFruitExtraRequirements(true, false)
	end
end

function CutterDensityHeightPickupFix:processPickupCutterArea(superFunc, workArea, dt)
	local spec = self.spec_cutter

	if spec.workAreaParameters.combineVehicle ~= nil then
		local lsx, lsy, lsz, lex, ley, lez, lineRadius = DensityMapHeightUtil.getLineByArea(workArea.start, workArea.width, workArea.height, true)

		for fruitType, state in pairs(spec.workAreaParameters.fruitTypesToUse) do
			if state then
				local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(fruitType)
				local fillTypeIndex = g_fruitTypeManager:getWindrowFillTypeIndexByFruitTypeIndex(fruitType)

				if fillTypeIndex == nil then
					fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(fruitDesc.name)
				end

				if fillTypeIndex ~= nil then
					local pickedUpLiters = -DensityMapHeightUtil.tipToGroundAroundLine(self, -math.huge, fillTypeIndex, lsx, lsy, lsz, lex, ley, lez, lineRadius, lineRadius, nil, false, nil)

					if self.isServer and pickedUpLiters > 0 then
						local literPerSqm = fruitDesc.literPerSqm
						local lastCutterArea = pickedUpLiters / (g_currentMission:getFruitPixelsToSqm() * literPerSqm)
						spec.currentInputFruitType = fruitType
						spec.useWindrow = true
						spec.currentInputFillType = fillTypeIndex
						spec.workAreaParameters.lastFruitType = fruitType
						spec.workAreaParameters.lastRealArea = spec.workAreaParameters.lastRealArea + lastCutterArea
						spec.workAreaParameters.lastThreshedArea = spec.workAreaParameters.lastThreshedArea + lastCutterArea
						spec.workAreaParameters.lastStatsArea = spec.workAreaParameters.lastStatsArea + lastCutterArea
						spec.workAreaParameters.lastArea = spec.workAreaParameters.lastArea + lastCutterArea
						spec.isWorking = true

						break
					end
				end
			end
		end
	end

	return spec.workAreaParameters.lastRealArea, spec.workAreaParameters.lastArea
end

function CutterDensityHeightPickupFix:getAllowCutterAIFruitRequirements(superFunc)
	return false
end
