local PARAM_BETWEEN = "between"
local PARAM_GREATER = "greater"
local PARAM_EQUAL = "equal"
local MODIFIER_3POINTS = "ppp"

function FSDensityMapUtil.updateRidgingHillerArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, allowedFruitTypes, maxFruitGrowthState, maxWeedGrowthState)
	local weedType = g_fruitTypeManager:getWeedFruitType()
	local numPixels = 0
	local totalNumPixels = 0

	if weedType ~= nil then
		local weedsModifiers = g_currentMission.densityMapModifiers.updateWeederArea
		local weedsModifier = weedsModifiers.modifier
		local weedsFilter = weedsModifiers.filter1
		local fruitsFilter = weedsModifiers.filter2
		local weed = weedType.weed
		maxWeedGrowthState = maxWeedGrowthState or weed.maxValue

		weedsModifier:setParallelogramWorldCoords(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, MODIFIER_3POINTS)
		weedsFilter:setValueCompareParams(PARAM_BETWEEN, 2, math.min(maxWeedGrowthState, weed.maxValue))

		for index, entry in pairs(g_currentMission.fruits) do
			local desc = g_fruitTypeManager:getFruitTypeByIndex(index)

			if desc.weed == nil and allowedFruitTypes[index] then
				fruitsFilter:resetDensityMapAndChannels(entry.id, desc.startStateChannel, desc.numStateChannels)
				fruitsFilter:setValueCompareParams(PARAM_BETWEEN, 1, math.min(maxFruitGrowthState, desc.numGrowthStates))

				local _, numP, totalNumP = weedsModifier:executeSet(0, weedsFilter, fruitsFilter)
				numPixels = numPixels + numP
				totalNumPixels = totalNumPixels + totalNumP
			end
		end
	end

	local desc = g_sprayTypeManager:getSprayTypeByIndex(SprayType.FERTILIZER)

	if desc ~= nil then
		local detailId = g_currentMission.terrainDetailId
		local groundType = desc.groundType
		local fertilizerModifiers = g_currentMission.densityMapModifiers.updateFertilizerArea
		local fertilizerModifier = fertilizerModifiers.modifier
		local fertilizerFilter = fertilizerModifiers.filter
		local fertilizerMaskFilter = fertilizerModifiers.maskFilter

		fertilizerModifier:setParallelogramWorldCoords(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, MODIFIER_3POINTS)

		local maskValue = g_currentMission.sprayMaxValue

		for index, entry in pairs(g_currentMission.fruits) do
			local desc = g_fruitTypeManager:getFruitTypeByIndex(index)

			if desc.weed == nil and allowedFruitTypes[index] then
				fertilizerMaskFilter:resetDensityMapAndChannels(entry.id, desc.startStateChannel, desc.numStateChannels)
				fertilizerMaskFilter:setValueCompareParams(PARAM_BETWEEN, 1, math.min(maxFruitGrowthState, desc.numGrowthStates))
				fertilizerFilter:setValueCompareParams(PARAM_GREATER, groundType)
				fertilizerModifier:executeSet(maskValue, fertilizerFilter, fertilizerMaskFilter)

				if groundType > 0 then
					fertilizerFilter:setValueCompareParams(PARAM_BETWEEN, 0, groundType - 1)
					fertilizerModifier:executeSet(maskValue, fertilizerFilter, fertilizerMaskFilter)
				end
			end
		end

		fertilizerFilter:setValueCompareParams(PARAM_EQUAL, maskValue)
		fertilizerMaskFilter:resetDensityMapAndChannels(detailId, g_currentMission.terrainDetailTypeFirstChannel, g_currentMission.terrainDetailTypeNumChannels)
		fertilizerMaskFilter:setValueCompareParams(PARAM_EQUAL, 0)
		fertilizerModifier:executeSet(0, fertilizerFilter, fertilizerMaskFilter)
		fertilizerModifier:resetDensityMapAndChannels(detailId, g_currentMission.sprayLevelFirstChannel, g_currentMission.sprayLevelNumChannels)
		fertilizerMaskFilter:resetDensityMapAndChannels(fertilizerModifier)
		fertilizerMaskFilter:setValueCompareParams(PARAM_BETWEEN, 0, g_currentMission.sprayLevelMaxValue - 1)
		fertilizerModifier:executeAdd(1, fertilizerFilter, fertilizerMaskFilter)
		fertilizerModifier:resetDensityMapAndChannels(detailId, g_currentMission.sprayFirstChannel, g_currentMission.sprayNumChannels)
		fertilizerModifier:executeSet(groundType, fertilizerFilter)
	end

	DensityMapHeightUtil.removeFromGroundByArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, FillType.GRASS_WINDROW)
	DensityMapHeightUtil.removeFromGroundByArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, FillType.DRYGRASS_WINDROW)
	DensityMapHeightUtil.removeFromGroundByArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, FillType.STRAW)

	return numPixels, totalNumPixels
end
